package main

import (
	"git.ufz.de/ecoepi/asfmm/globals"
	"git.ufz.de/ecoepi/asfmm/obs"
	"git.ufz.de/ecoepi/asfmm/params"
	"git.ufz.de/ecoepi/asfmm/sys"
	"github.com/mlange-42/ark-pixel/monitor"
	"github.com/mlange-42/ark-pixel/window"
	"github.com/mlange-42/ark-tools/app"
	"github.com/mlange-42/ark-tools/system"
	"github.com/mlange-42/ark/ecs"
)

func main() {
	model := app.New() //.Seed(42)

	model.TPS = 60 // Limit model speed

	p := params.DefaultParams()
	ecs.AddResource(&model.World, &p)
	g := globals.WorldGlobals{}
	ecs.AddResource(&model.World, &g)

	model.AddSystem(&sys.InitHerds{})
	model.AddSystem(&sys.ReleaseVirusCenter{})
	model.AddSystem(&sys.ProgressDisease{})
	model.AddSystem(&system.FixedTermination{Steps: p.World.SimulationDays})

	model.AddUISystem((&window.Window{Bounds: window.B(5, 30, 600, 400)}).
		With(
			&monitor.Monitor{SampleInterval: 30},
			&monitor.Controls{},
		))

	model.AddUISystem((&window.Window{Bounds: window.B(3, 460, 600, 500)}).
		With(&monitor.Resources{}))

	model.AddUISystem((&window.Window{Bounds: window.B(610, 30, 900, 900)}).
		With(&obs.HerdMap{}))

	window.Run(model)

}
