package obs

import (
	"image/color"

	"git.ufz.de/ecoepi/asfmm/comps"
	"git.ufz.de/ecoepi/asfmm/params"
	"github.com/gopxl/pixel/v2"
	"github.com/gopxl/pixel/v2/backends/opengl"
	"github.com/gopxl/pixel/v2/ext/imdraw"
	"github.com/mlange-42/ark-pixel/window"
	"github.com/mlange-42/ark/ecs"
)

// HerdMap drawer
type HerdMap struct {
	paramsRes  ecs.Resource[params.Parameters]
	filterHerd *ecs.Filter3[comps.HerdPosition, comps.HerdStock, comps.HerdDiseaseStatus]

	herdColors    []color.NRGBA
	diseaseColors []color.NRGBA
	drawer        imdraw.IMDraw
}

// Initialize the system
func (s *HerdMap) Initialize(w *ecs.World, win *opengl.Window) {
	s.filterHerd = s.filterHerd.New(w)
	s.paramsRes = ecs.NewResource[params.Parameters](w)

	s.herdColors = make([]color.NRGBA, comps.NumHerdTypes)
	s.herdColors[comps.Finisher] = color.NRGBA{255, 255, 255, 255} // white
	s.herdColors[comps.Sow] = color.NRGBA{0, 255, 255, 255}        // cyan
	s.herdColors[comps.Piglet] = color.NRGBA{0, 0, 255, 255}       //	 blue

	s.diseaseColors = make([]color.NRGBA, comps.NumDiseaseStatus)
	s.diseaseColors[comps.Exposed] = color.NRGBA{255, 165, 0, 255}  // orange
	s.diseaseColors[comps.Infectious] = color.NRGBA{255, 0, 0, 255} //red

	s.drawer = *imdraw.New(nil)
}

// Update the system
func (s *HerdMap) Update(w *ecs.World) {
}

// UpdateInputs handles input events of the previous frame update.
func (s *HerdMap) UpdateInputs(w *ecs.World, win *opengl.Window) {
}

// Draw the system
func (s *HerdMap) Draw(w *ecs.World, win *opengl.Window) {
	mapSize := s.paramsRes.Get().World.SquareSideLength
	scale := window.Scale(win, float64(mapSize), float64(mapSize))

	dr := &s.drawer

	// Draw all herds with their type color
	queryHerd := s.filterHerd.Query()
	for queryHerd.Next() {
		pos, herd, status := queryHerd.Get()

		if status.Status != comps.Susceptible {
			dr.Color = s.diseaseColors[status.Status]
		} else {
			dr.Color = s.herdColors[herd.Type]
		}
		sx, sy := 0.5, 0.5
		if herd.Type == comps.Piglet {
			sx, sy = 0.25, 0.25
		}
		dr.Push(pixel.V((pos.X-sx)*scale, (pos.Y-sy)*scale), pixel.V((pos.X+sx)*scale, (pos.Y+sy)*scale))
		dr.Rectangle(0)
		dr.Reset()
	}
	dr.Draw(win)
	dr.Clear()
}
