package params

// WorldParameters holds the parameters for the simulated world.
type WorldParameters struct {
	SquareSideLength int     // km
	FarmDensity      float64 // per sqkm

	SimulationDays int64 // number of days to simulate
}

// HerdParameters holds the parameters for the herds in the simulation.
type HerdParameters struct {
	FractionSowPiglet         float64 // proportion of sow and piglet herds in the entire world population
	FinishersPerFarm          int
	SowsPerFarm               int
	PigletsPerFarm            int
	RandomnessRangeForNumPigs float64 // +/- proportion around the mean

	FinisherPenSize int
	SowPenSize      int
	PigletPenSize   int
}

// DiseaseParameters holds the parameters for the disease in the simulation.
type DiseaseParameters struct {
	VirusReleaseDay uint64
	IncubationDays  uint8 // days
}

// Parameters holds the parameters for the simulation.
type Parameters struct {
	World   WorldParameters
	Herd    HerdParameters
	Disease DiseaseParameters
}

// DefaultParams returns the default parameters for the simulation.
func DefaultParams() Parameters {
	return Parameters{
		World: WorldParameters{
			SquareSideLength: 200,
			FarmDensity:      1,
			SimulationDays:   365,
		},
		Herd: HerdParameters{
			FractionSowPiglet: 0.25,

			FinishersPerFarm:          1000,
			SowsPerFarm:               400,
			PigletsPerFarm:            1600,
			RandomnessRangeForNumPigs: 0.5,

			FinisherPenSize: 10,
			SowPenSize:      1,
			PigletPenSize:   10,
		},
		Disease: DiseaseParameters{
			VirusReleaseDay: 50,
			IncubationDays:  4,
		},
	}
}
