package sys

import (
	"math"
	"math/rand/v2"

	"git.ufz.de/ecoepi/asfmm/comps"
	"git.ufz.de/ecoepi/asfmm/params"
	"github.com/mlange-42/ark-tools/resource"
	"github.com/mlange-42/ark/ecs"
)

// InitHerds initializes the herds in the simulation.
type InitHerds struct {
}

// Initialize herds in the model world.
func (s *InitHerds) Initialize(w *ecs.World) {

	randSrc := ecs.GetResource[resource.Rand](w)
	rng := rand.New(randSrc)

	p := ecs.GetResource[params.Parameters](w)
	farmPigletSowBuilder := ecs.NewMap5[comps.HerdID, comps.HerdPosition, comps.HerdStock, comps.HerdDiseaseStatus, comps.LinkedHerd](w)
	farmFinisherBuilder := ecs.NewMap4[comps.HerdID, comps.HerdPosition, comps.HerdStock, comps.HerdDiseaseStatus](w)
	linkedHerd := ecs.NewMap[comps.LinkedHerd](w)

	// NOTE: truncated!:
	numFarms := int(float64(p.World.SquareSideLength*p.World.SquareSideLength) * p.World.FarmDensity)

	createdFarms := 0

	for createdFarms < numFarms {

		testPos := comps.HerdPosition{
			X: rng.Float64() * float64(p.World.SquareSideLength),
			Y: rng.Float64() * float64(p.World.SquareSideLength),
		}
		// OPEN: test in Cpp; while loop

		if rng.Float64() < p.Herd.FractionSowPiglet {

			farm1 := farmPigletSowBuilder.NewEntity(
				&comps.HerdID{Value: createdFarms},
				&testPos,
				&comps.HerdStock{
					Type:    comps.Sow,
					NumPigs: CalculateNumPigs(float64(p.Herd.SowsPerFarm), p.Herd.SowPenSize, rng, p.Herd.RandomnessRangeForNumPigs),
				},
				&comps.HerdDiseaseStatus{Status: comps.Susceptible},
				&comps.LinkedHerd{},
			)
			createdFarms++

			farm2 := farmPigletSowBuilder.NewEntity(
				&comps.HerdID{Value: createdFarms},
				&testPos,
				&comps.HerdStock{
					Type:    comps.Piglet,
					NumPigs: CalculateNumPigs(float64(p.Herd.PigletsPerFarm), p.Herd.PigletPenSize, rng, p.Herd.RandomnessRangeForNumPigs),
				},
				&comps.HerdDiseaseStatus{Status: comps.Susceptible},
				&comps.LinkedHerd{Herd: farm1},
			)
			linked := linkedHerd.Get(farm1)
			linked.Herd = farm2
			createdFarms++
			continue
		}
		_ = farmFinisherBuilder.NewEntity(
			&comps.HerdID{Value: createdFarms},
			&testPos,
			&comps.HerdStock{
				Type:    comps.Finisher,
				NumPigs: CalculateNumPigs(float64(p.Herd.FinishersPerFarm), p.Herd.FinisherPenSize, rng, p.Herd.RandomnessRangeForNumPigs),
			},
			&comps.HerdDiseaseStatus{Status: comps.Susceptible},
		)
		createdFarms++

	}
}

// Update herds in the model world.
func (s *InitHerds) Update(w *ecs.World) {

}

// Finalize herds in the model world.
func (s *InitHerds) Finalize(w *ecs.World) {
	// fmt.Println("NumFarms:", s.NumFarms)
}

// CalculateNumPigs computes the number of pigs by introducing randomness around a mean value, scaled
// by a rangeFactor, and ensures the result is rounded to the nearest multiple of the specified
// penSize. This guarantees that the number of pigs fits evenly into pens of the given size.
func CalculateNumPigs(mean float64, penSize int, rng *rand.Rand, rangeFactor float64) int {
	// Calculate the number of pigs introducing randomness
	numPigs := mean + (2*rangeFactor*rng.Float64()-rangeFactor)*mean
	// Round to the nearest multiple of penSize
	return int(math.Round(numPigs/float64(penSize)) * float64(penSize))
}
