package sys

import (
	"git.ufz.de/ecoepi/asfmm/comps"
	"git.ufz.de/ecoepi/asfmm/params"
	"github.com/mlange-42/ark-tools/resource"
	"github.com/mlange-42/ark/ecs"
)

// ProgressDisease is a system that progresses the disease in the simulation.
type ProgressDisease struct {
	tickRes             ecs.Resource[resource.Tick]
	params              ecs.Resource[params.Parameters]
	filterDiseaseStatus *ecs.Filter1[comps.HerdDiseaseStatus]
}

// Initialize disease progression
func (s *ProgressDisease) Initialize(w *ecs.World) {
	s.tickRes = ecs.NewResource[resource.Tick](w)
	s.params = ecs.NewResource[params.Parameters](w)
	s.filterDiseaseStatus = s.filterDiseaseStatus.New(w)

}

// Update disease progression
func (s *ProgressDisease) Update(w *ecs.World) {

	incubationDays := s.params.Get().Disease.IncubationDays
	tick := uint64(s.tickRes.Get().Tick)

	queryStatus := s.filterDiseaseStatus.Query()
	for queryStatus.Next() {
		status := queryStatus.Get()
		switch status.Status {
		case comps.Susceptible:
			// Check for exposure to infectious herds
			// get entities grid number (int|int)
			// create all combis as a map (-1,0,+1, if grid numbering is int|int)--> similar to Rs expand.grid
			// loop over combi map and infect entities by probability
			// for i := range combis {
			// 	grid := &combis.x[i] || &combis.y[i]
		case comps.Exposed:
			if status.DaysIncubated == incubationDays {
				status.Status = comps.Infectious
				status.DaysIncubated = 0
			} else {
				status.DaysIncubated = uint8((tick - status.TickOfExposure))
			}
		}
	}
}

// Finalize disease progression
func (s *ProgressDisease) Finalize(w *ecs.World) {
}
