package sys_test

import (
	"math"
	"testing"

	"git.ufz.de/ecoepi/asfmm/comps"
	"git.ufz.de/ecoepi/asfmm/globals"
	"git.ufz.de/ecoepi/asfmm/params"
	"git.ufz.de/ecoepi/asfmm/sys"
	"github.com/mlange-42/ark-tools/app"
	"github.com/mlange-42/ark-tools/system"
	"github.com/mlange-42/ark/ecs"
	"github.com/stretchr/testify/assert"
)

func TestReleaseVirusCenter(t *testing.T) {
	model := app.New() //.Seed(42)
	p := params.DefaultParams()
	ecs.AddResource(&model.World, &p)
	g := globals.WorldGlobals{}
	ecs.AddResource(&model.World, &g)

	model.AddSystem(&sys.InitHerds{})
	model.AddSystem(&sys.ReleaseVirusCenter{})
	model.AddSystem(&system.FixedTermination{Steps: p.World.SimulationDays})

	model.Run()

	farmFilter := ecs.NewFilter2[comps.HerdPosition, comps.HerdDiseaseStatus](&model.World)

	// compute min distance among all herds
	centerX, centerY := float64(p.World.SquareSideLength/2), float64(p.World.SquareSideLength/2)
	var minDistance float64 = math.MaxFloat64

	query := farmFilter.Query()
	for query.Next() {
		position, _ := query.Get()
		distance := math.Sqrt(math.Pow(position.X-centerX, 2) + math.Pow(position.Y-centerY, 2))
		if distance < minDistance {
			minDistance = distance
		}
	}

	// find exposed herds (only one or two after InitDiseaseCenter) and check if their distance equals minDistance
	query = farmFilter.Query()
	for query.Next() {
		position, status := query.Get()
		if status.Status != comps.Exposed {
			continue
		}
		distance := math.Sqrt(math.Pow(position.X-centerX, 2) + math.Pow(position.Y-centerY, 2))
		assert.Equal(t, minDistance, distance, "Exposed herd %v is not the closest to the center")
	}

	assert.True(t, g.VirusReleased, "Virus was not released in the world?")

}
